require 'yaml'

module Envvy
  class Engine < Rails::Engine
    attr_accessor :env

    class << self
      attr_accessor :config_path

      def config_path
        @config_path || File.join(Rails.root, "config", "env_vars.yml")
      end
    end

    initializer "env-loader.precompile" do |app|
      app.config.assets.paths << root.join('assets', 'javascripts').to_s
    end

    initializer "envvy.register_preprocessor", :after => "sprockets.environment" do
      if Rails.application.assets
        Rails.application.assets.register_preprocessor "application/javascript", :"js_env" do |context, source|
          if context.logical_path == "js_env" && File.exist?(self.class.config_path)
            context.depend_on(File.expand_path(self.class.config_path))
          end
          source
        end
      end
    end

    config.before_initialize do
      set_env self.config_path

      if Rails.env.development?
        require 'filewatch/watch'
        conf_watcher = FileWatch::Watch.new
        conf_watcher.watch(self.config_path)

        Thread.new do
          conf_watcher.subscribe(1,5) do |_status, path|
            set_env path
          end
        end
      end

      self.get("config") do |config|
        config.each do |k, v|
          ENV[k.to_s.upcase] ||= v
        end
      end
    end

    def set_env file
      self.env = {"js_config" => {}, "config" => {}}
      self.env.merge! YAML.load(File.read(file)) if File.exist?(file)

      if env[Rails.env]
        self.env.deep_merge!(self.env[Rails.env])
      end
    end

    def get(key, &block)
      if block_given?
        yield self.env[key]
      else
        return self.env[key]
      end
    end

    private

    def merge_current_env!(hash)
      rails_envs.each do |rails_env|
        if Rails.env == rails_env && hash[rails_env]
          hash.merge! hash[rails_env]
        end
        hash.delete rails_env
      end
    end

    def rails_envs
      @rails_envs ||=  Dir.glob(Rails.root + "config/environments/*.rb")
                          .map { |filename| File.basename(filename, ".rb") }
    end
  end
end
